@@ -6,6 +6,8 @@ use k8s_openapi::apimachinery::pkg::apis::meta::v1 as metav1;
 use serde::{Deserialize, Serialize};
 use serde_with::skip_serializing_none;
 
+pub use cluster::ClusterIpFamily;
+pub use cluster::InvalidIpFamily;
 pub use cluster::*;
 pub use common::*;
 pub use condition::*;
