@@ -2,7 +2,11 @@ use kube::CustomResource;
 
 use super::*;
 
+pub use ip_family::ClusterIpFamily;
+pub use ip_family::InvalidIpFamily;
+
 mod impls;
+mod ip_family;
 
 /// ClusterFinalizer is the finalizer used by the cluster controller to
 /// cleanup the cluster resources when a Cluster is being deleted.
@@ -17,7 +21,9 @@ pub const CLUSTER_FINALIZER: &str = "cluster.cluster.x-k8s.io";
     group = "cluster.x-k8s.io",
     version = "v1beta1",
     kind = "Cluster",
-    plural = "clusters"
+    plural = "clusters",
+    shortname = "cl",
+    status = "ClusterStatus"
 )]
 #[kube(namespaced)]
 #[kube(schema = "disabled")]
@@ -208,11 +214,11 @@ pub struct ClusterStatus {
 
     /// InfrastructureReady is the state of the infrastructure provider.
     // +optional
-    pub infrastructure_ready: bool,
+    pub infrastructure_ready: Option<bool>,
 
     /// ControlPlaneReady defines if the control plane is ready.
     // +optional
-    pub control_plane_ready: bool,
+    pub control_plane_ready: Option<bool>,
 
     /// Conditions defines current service state of the cluster.
     // +optional
@@ -249,107 +255,7 @@ pub struct ApiEndpoint {
 // +kubebuilder:printcolumn:name="Age",type="date",JSONPath=".metadata.creationTimestamp",description="Time duration since creation of Cluster"
 // +kubebuilder:printcolumn:name="Version",type="string",JSONPath=".spec.topology.version",description="Kubernetes version associated with this Cluster"
 
-/// Cluster is the Schema for the clusters API.
-#[derive(Clone, Debug, Default, Serialize, Deserialize)]
-pub struct Cluster  {
-    pub metadata: metav1::ObjectMeta,
-    pub spec: Option<ClusterSpec>,
-    pub status: Option<ClusterStatus>,
-}
 
-// GetConditions returns the set of conditions for this object.
-func (c *Cluster) GetConditions() Conditions {
-    return c.Status.Conditions
-}
-
-// SetConditions sets the conditions on this object.
-func (c *Cluster) SetConditions(conditions Conditions) {
-    c.Status.Conditions = conditions
-}
-
-// GetIPFamily returns a ClusterIPFamily from the configuration provided.
-func (c *Cluster) GetIPFamily() (ClusterIPFamily, error) {
-    var podCIDRs, serviceCIDRs []string
-    if c.Spec.ClusterNetwork != nil {
-        if c.Spec.ClusterNetwork.Pods != nil {
-            podCIDRs = c.Spec.ClusterNetwork.Pods.CIDRBlocks
-        }
-        if c.Spec.ClusterNetwork.Services != nil {
-            serviceCIDRs = c.Spec.ClusterNetwork.Services.CIDRBlocks
-        }
-    }
-    if len(podCIDRs) == 0 && len(serviceCIDRs) == 0 {
-        return IPv4IPFamily, nil
-    }
-
-    podsIPFamily, err := ipFamilyForCIDRStrings(podCIDRs)
-    if err != nil {
-        return InvalidIPFamily, fmt.Errorf("pods: %s", err)
-    }
-    if len(serviceCIDRs) == 0 {
-        return podsIPFamily, nil
-    }
-
-    servicesIPFamily, err := ipFamilyForCIDRStrings(serviceCIDRs)
-    if err != nil {
-        return InvalidIPFamily, fmt.Errorf("services: %s", err)
-    }
-    if len(podCIDRs) == 0 {
-        return servicesIPFamily, nil
-    }
-
-    if podsIPFamily == DualStackIPFamily {
-        return DualStackIPFamily, nil
-    } else if podsIPFamily != servicesIPFamily {
-        return InvalidIPFamily, errors.New("pods and services IP family mismatch")
-    }
-
-    return podsIPFamily, nil
-}
-
-func ipFamilyForCIDRStrings(cidrs []string) (ClusterIPFamily, error) {
-    if len(cidrs) > 2 {
-        return InvalidIPFamily, errors.New("too many CIDRs specified")
-    }
-    var foundIPv4 bool
-    var foundIPv6 bool
-    for _, cidr := range cidrs {
-        ip, _, err := net.ParseCIDR(cidr)
-        if err != nil {
-            return InvalidIPFamily, fmt.Errorf("could not parse CIDR: %s", err)
-        }
-        if ip.To4() != nil {
-            foundIPv4 = true
-        } else {
-            foundIPv6 = true
-        }
-    }
-    switch {
-    case foundIPv4 && foundIPv6:
-        return DualStackIPFamily, nil
-    case foundIPv4:
-        return IPv4IPFamily, nil
-    case foundIPv6:
-        return IPv6IPFamily, nil
-    default:
-        return InvalidIPFamily, nil
-    }
-}
-
-// ClusterIPFamily defines the types of supported IP families.
-type ClusterIPFamily int
-
-// Define the ClusterIPFamily constants.
-const (
-    InvalidIPFamily ClusterIPFamily = iota
-    IPv4IPFamily
-    IPv6IPFamily
-    DualStackIPFamily
-)
-
-func (f ClusterIPFamily) String() string {
-    return [...]string{"InvalidIPFamily", "IPv4IPFamily", "IPv6IPFamily", "DualStackIPFamily"}[f]
-}
 
 // +kubebuilder:object:root=true
 
